unit DocView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry, WinSystem, stdctrls;

type
  TDocView = class(TComponent)
  private
    { Private declarations }
    FDocExtension: String;
    FDocDescription: String;
    Registry: TRegistry;
  protected
    { Protected declarations }
  public
    { Public declarations }
    function  ShellReg(Form: TForm; appPath: String; iconID: Integer;  paramNo: Integer): Boolean;
    function  ShellUnreg(Form: TForm): Boolean;
    function  Serialize(comp: TComponent; filename: String): Boolean;
    function  Unserialize(comp: TComponent; filename: String): Boolean;
    function  CheckExtension(Form: TForm): Boolean;
    procedure SaveAsText(lines: TStrings; filename: String);
    procedure AutoLoadFile(comp: TComponent);
    procedure AutoLoadTextFile(var memo: TMemo);
    procedure AutoRun(appTitle: String; appExeName: String; runonce: Boolean);
    constructor Create(Owner: TComponent); override;

  published
    { Published declarations }
    property AppExtension: String read FDocExtension write FDocExtension;
    property AppDescription: String read FDocDescription write FDocDescription;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Alexis Ros', [TDocView]);
end;


constructor TDocView.Create(Owner: TComponent);
begin
    inherited Create(Owner);

end;

function TDocView.ShellReg(Form: TForm; appPath: String; iconID: Integer;  paramNo: Integer): Boolean;
var
    mainItem: String;
    nextItem: String;
    runParam: String;
    msg     : String;
begin
    if(FDocExtension = '') then
    begin
        MessageBox(Form.Handle, 'Document extension is required', 'Doc Extension', MB_ICONSTOP);
        Result := false;
        Exit;
    end
    else if(FDocDescription = '') then
    begin
        MessageBox(Form.Handle, 'Document description is required', 'Doc Extension', MB_ICONSTOP);
        Result := false;
        Exit;
    end;
    mainItem := '.'+Trim(FDocExtension);
    nextItem := 'Software\Classes\' + Trim(FDocDescription);

    try

        Registry := TRegistry.Create();
        Registry.RootKey := HKEY_CLASSES_ROOT;
        if not (Registry.KeyExists(mainItem)) then
        begin
            Registry.OpenKey(mainItem, true);
            Registry.WriteString('', FDocDescription);
            Registry.CloseKey();

            Registry.RootKey := HKEY_LOCAL_MACHINE;
            Registry.OpenKey(nextItem, true);
            Registry.WriteString('', FDocDescription + ' Document');
            Registry.CloseKey();

            nextItem := 'Software\Classes\' + FDocDescription + '\DefaultIcon';
            Registry.OpenKey(nextItem, true);
            Registry.WriteString('', appPath + ',' + Trim(IntToStr(iconID)));
            Registry.CloseKey();

            nextItem := 'Software\Classes\' + FDocDescription + '\Shell\Open\Command';
            runParam := appPath + ' "%'+Trim(IntToStr(paramNo)) + '"';
            Registry.OpenKey(nextItem, true);
            Registry.WriteString('', runParam);
            Registry.CloseKey();
            Registry.Free();
            Result := true;
        end
        else
        begin
            msg := 'File extension ' + self.AppExtension + ' is already registered with another application...' + #10 + #10 +'Do you want to replace it with this software?';
            if( MessageBox(Form.Handle, PChar(msg), 'Register', MB_YESNO+MB_ICONQUESTION) = IDYES ) then
            begin
                Registry := TRegistry.Create();
                Registry.RootKey := HKEY_CLASSES_ROOT;
                Registry.OpenKey(mainItem, true);
                Registry.WriteString('', FDocDescription);
                Registry.CloseKey();

                Registry.RootKey := HKEY_LOCAL_MACHINE;
                Registry.OpenKey(nextItem, true);
                Registry.WriteString('', FDocDescription + ' Document');
                Registry.CloseKey();

                nextItem := 'Software\Classes\' + FDocDescription + '\DefaultIcon';
                Registry.OpenKey(nextItem, true);
                Registry.WriteString('', appPath + ',' + Trim(IntToStr(iconID)));
                Registry.CloseKey();
                nextItem := 'Software\Classes\' + FDocDescription + '\Shell\Open\Command';
                runParam := appPath + ' "%'+Trim(IntToStr(paramNo)) + '"';
                Registry.OpenKey(nextItem, true);
                Registry.WriteString('', runParam);
                Registry.CloseKey();
                Registry.Free();
                Result := true;
            end
            else
            begin
                Registry.Free();
                Result := false;
            end;
        end;
    except

        Registry.Free();
        Result := false;
    end;


end;


function  TDocView.ShellUnreg(Form: TForm): Boolean;
var
    mainItem: String;
    nextItem: String;
begin
    if(FDocExtension = '') then
    begin
        MessageBox(Form.Handle, 'Document extension is required', 'Doc Extension', MB_ICONSTOP);
        Result := false;
        Exit;
    end
    else if(FDocDescription = '') then
    begin
        MessageBox(Form.Handle, 'Document description is required', 'Doc Extension', MB_ICONSTOP);
        Result := false;
        Exit;

    end;
    mainItem := '.'+Trim(FDocExtension);
    nextItem := 'Software\Classes\' + Trim(FDocDescription);


    try

        Registry := TRegistry.Create();
        Registry.RootKey := HKEY_CLASSES_ROOT;
        if(Registry.KeyExists(mainItem)) then
        begin
            Registry.DeleteKey(mainItem);
            Registry.CloseKey();

            Registry.RootKey := HKEY_LOCAL_MACHINE;
            Registry.DeleteKey(nextItem);
            Registry.CloseKey();
            Registry.Free();
            Result := true;
        end
        else
        begin
            MessageBox(Form.Handle, 'File extension is not registered', 'ShellReg', MB_ICONSTOP);
            Registry.Free;
            Result := false;
        end;

    except
        Registry.Free();
        Result := false;
    end;


end;


function TDocView.Serialize(comp: TComponent; filename: String): Boolean;
var
    fs: TFileStream;
begin
        try

            fs := TFileStream.Create(filename, fmCreate);
            fs.WriteComponentRes('DELPHI5DOCVIEW', comp);
            fs.Free();
            Result := true;
        except
            Result := false;
        end;

end;
procedure TDocView.SaveAsText(lines: TStrings; filename: String);
var
    list: TStringList;
begin
    list:= TStringList.Create();
    list.AddStrings(lines);
    list.SaveToFile(filename);
    list.Free();

end;


function TDocView.Unserialize(comp: TComponent; filename: String): Boolean;
var
    fs: TFileStream;
begin
    fs := TFileStream.Create(filename, fmOpenRead);
    fs.ReadComponentRes(comp);
    fs.Free();
    Result := true;

end;
function TDocView.CheckExtension(Form: TForm): Boolean;
var
    mainItem: String;
    nextItem: String;
begin
    if(Trim(FDocExtension) = '') then
    begin
        MessageBox(Form.Handle, 'You need to supply the file extension', 'Required', MB_ICONSTOP);
        Result := false;
        Exit;
    end;
    mainItem := '.'+ Trim(FDocExtension);
    nextItem := 'Software\Classes\' + Trim(FDocDescription);
    Result:= true;
    try

        Registry := TRegistry.Create();
        Registry.RootKey := HKEY_CLASSES_ROOT;
        if(Registry.KeyExists(mainItem)) then
        begin
            Registry.Free();
            Result := true;
        end
        else
        begin
            Registry.Free();
            Result := false;
        end;

    except
        Registry.Free();
    end;

end;
procedure TDocView.AutoRun(appTitle: String; appExeName: String; runonce: Boolean);
var
    ws: TWinSystem;
begin
    ws := TWinSystem.Create(self);
    ws.RunOnStartup(appTitle, appExeName, runonce);
    ws.Free();
end;
procedure TDocView.AutoLoadFile(comp: TComponent);
begin
       if ParamCount() > 0 then
       begin
          self.Unserialize(comp, ParamStr(1));
       end;
end;

procedure TDocView.AutoLoadTextFile(var memo: TMemo);
begin
        if ParamCount() > 0 then
            memo.Lines.LoadFromFile(ParamStr(1));
end;

end.
