unit FlashingLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, extctrls;

type
  TLimitReachedEvent =
        procedure(Sender: TObject; var Stop: Boolean) of object;
  TFlashingLabel = class(TCustomLabel)
  private
    { Private declarations }
    FFlashEnabled    : Boolean;
    FFlashLimit      : Integer;
    FFlashRate       : Integer;
    FOnLimitReached  : TLimitReachedEvent;
    FlashCount       : Integer;
    Timer            : TTimer;

  protected
    { Protected declarations }
    procedure SetFlashEnabled(AFlashEnabled: Boolean);
    procedure SetFlashRate(AFlashRate: Integer);
    procedure OnTimer(Sender: TObject); virtual;
    procedure DoLimitReached; virtual;
    procedure Click; override;
  public
    constructor Create(AOwner: TComponent); override;
    { Public declarations }
  published
    { Published declarations }
    property FlashEnabled: Boolean
        read FFlashEnabled write SetFlashEnabled default True;
    property FlashRate: Integer
        read FFlashRate write SetFlashRate default 800;
    property FlashLimit: Integer
        read FFlashLimit write FFlashLimit default 0;
    property OnLimitReached: TLimitReachedEvent
        read FOnLimitReached write FOnLimitReached;
    {All the properties of TCustomLabel redeclared}
    property Align;
    property Alignment;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Alexis Ros', [TFlashingLabel]);
end;

constructor TFlashingLabel.Create(AOwner: TComponent);
begin
        inherited;
        FFlashEnabled := true;
        FFlashRate := 800;
        FFlashLimit := 0;
        FlashCount := 0;
        Timer := TTimer.Create(self);
        Timer.Interval := FFlashRate;
        Timer.OnTimer := OnTimer;
        Transparent := true;
end;

procedure TFlashingLabel.DoLimitReached;
var
        Stop: Boolean;
begin
        Stop := true;
        if Assigned(FOnLimitReached) then
                FOnLimitReached(Self, Stop);
        self.FlashEnabled := not Stop;
end;

procedure TFlashingLabel.OnTimer(Sender: TObject);
begin
        if csDesigning in ComponentState then begin
                Timer.Enabled := false;
                Exit;
        end;
        Visible := not Visible;
        if (FFlashLimit <> 0) and Visible then begin
                Inc(FlashCount);
                if FlashCount >= FFlashLimit then begin
                   FlashCount := 0;
                   DoLimitReached;
                end;
        end;
end;

procedure TFlashingLabel.Click;
begin
    MessageBeep(0);
    inherited;
end;

procedure TFlashingLabel.SetFlashEnabled(AFlashEnabled: Boolean);
begin
        FFlashEnabled := AFlashEnabled;
        if csDesigning in ComponentState then
                Exit;
        Timer.Enabled := FFlashEnabled;
        if not FFlashEnabled then
                Visible := true;
end;

procedure TFlashingLabel.SetFlashRate(AFlashRate: Integer);
begin
        FFlashRate := AFlashRate;
        Timer.Interval := AFlashRate;
end;

end.
