unit WinSystem;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ShellApi, Registry;

type
  TWinSystem = class(TComponent)
  private
    { Private declarations }
    FWinType: String;
    FMajor: String;
    FMinor: String;
    FBuild: String;
    FOtherInfo: String;
    FComputerName: String;
    FUserId: String;
    FColors: String;
    FSmallFonts: Boolean;
    FCompanyName: String;
    FFileDescription: String;
    FFileVersion: String;
    FInternalName: String;
    FLegalCopyright: String;
    FLegalTradeMarks: String;
    FOriginalFilename: String;
    FProductName: String;
    FProductVersion: String;
    FComments: String;

    os: OSVERSIONINFO;
    function SetPrivilege(
      sPrivilegeName : string;
      bEnabled : boolean )
        : boolean;

  protected
    { Protected declarations }
    procedure HandleIOError();
  public
    { Public declarations }
    constructor Create(Owner: TComponent); override;
    procedure GetOSInfo();
    procedure GetComputerInfo();
    procedure GetColorsCount;
    procedure GoWeb(Form: TForm; URL: String);
    procedure GetFontSize();
    procedure AppMessage(
      var Msg : TMsg;
      var bHandled : boolean );
    procedure RunOnStartup(
      sProgTitle,
      sCmdLine    : string;
      bRunOnce    : boolean );
    procedure RevokeRunOnStartup(sProgTitle: String);
    procedure EnableScreenSaver();
    procedure DisableScreenSaver();
    procedure RecycleFiles(Form: TForm; sFiles: String);
    procedure CopyFlyingFiles(Form: TForm; sFrom: String; sTo: String);
    procedure MoveFlyingFiles(Form: TForm; sFrom: String; sTo: String);
    procedure GetExeVersion(sFilename: String);
    function GetDiskDriveType(sDrive: String): String;
    function WinExit( iFlags : integer ) : boolean;
    function GetFileSize(filename: String): Longint;


  published
    { Published declarations }
    property WindowsType: String read FWinType write FWinType;
    property MajorVersion: String read FMajor write FMajor;
    property MinorVersion: String read FMinor write FMinor;
    property BuildNumber: String read FBuild write FBuild;
    property OtherInfo: String read FOtherInfo write FOtherInfo;
    property ComputerName: String read FComputerName write FComputerName;
    property MaxColors: String read FColors write FColors;
    property UserId: String read FUserId write FUserId;
    property UsingSmallFonts: Boolean read FSmallFonts write FSmallFonts;
    property CompanyName: String read FCompanyName write FCompanyName;
    property FileDescription: String read FFileDescription write FFileDescription;
    property FileVersion: String read FFileVersion write FFileVersion;
    property InternalName: String read FInternalName write FInternalName;
    property LegalCopyright: String read FLegalCopyright write FLegalCopyright;
    property LegalTradeMarks: String read FLegalTradeMarks write FLegalTradeMarks;
    property OriginalFilename: String read FOriginalFilename write FOriginalFilename;
    property ProductName: String read FProductName write FProductName;
    property ProductVersion: String read FProductVersion write FProductVersion;
    property Comments: String read FComments write FComments;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Alexis Ros', [TWinSystem]);
end;

constructor TWinSystem.Create(Owner: TComponent);
begin
  inherited Create(Owner);  // Initialize inherited parts
  With self do begin
     GetOSInfo;
     GetComputerInfo;
     GetColorsCount;
     GetFontSize;
  end;
end;

procedure TWinSystem.GetOSInfo();
var
    x: Boolean;
begin
    os.dwOSVersionInfoSize :=  SizeOf(os);
    x := GetVersionEx(os);
    if x = true then
    begin
        FMajor := IntToStr(os.dwMajorVersion);
        FMinor := IntToStr(os.dwMinorVersion);
        case os.dwPlatformId of
         VER_PLATFORM_WIN32_WINDOWS:
            FWinType := 'Windows 95/98';
         VER_PLATFORM_WIN32_NT:
            FWinType := 'Windows NT';
        else
            FWinType := 'Windows 2000';
        end;
        FBuild := IntToStr(os.dwBuildNumber);
        FOtherInfo := os.szCSDVersion;
    end;
end;

procedure TWinSystem.GetComputerInfo();
var
    szBuffer: array[0..MAX_COMPUTERNAME_LENGTH + 1] of char;
    nSize : Cardinal;
    NameBuff: array[0..80] of Char;
    SizeBuff: LongWord;
begin
    nSize := High(szBuffer);
    GetComputerName(szBuffer, nSize);
    FComputerName := szBuffer;
    SizeBuff := SizeOf(NameBuff);
    GetUserName(NameBuff, SizeBuff);
    FUserId := NameBuff;

end;

procedure TWinSystem.GoWeb(Form: TForm; URL: String);
begin
    ShellExecute(Form.Handle,'open', PChar(URL), nil, nil, SW_SHOWMAXIMIZED);
end;


procedure TWinSystem.GetColorsCount();
var
  h : hDC;
begin
  FColors := IntToStr(0);
  h := GetDC( 0 );
  try
    FColors := IntToStr (
      1 shl
      (
        GetDeviceCaps( h, PLANES ) *
        GetDeviceCaps( h, BITSPIXEL )
      ));
  finally
    ReleaseDC( 0, h );
  end;
end;

function TWinSystem.SetPrivilege(
  sPrivilegeName : string;
  bEnabled : boolean )
    : boolean;
var
  TPPrev,
  TP         : TTokenPrivileges;
  Token      : THandle;
  dwRetLen   : DWord;
begin
  Result := False;

  OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY,Token );

  TP.PrivilegeCount := 1;
  if( LookupPrivilegeValue(
        Nil,
        PChar( sPrivilegeName ),
        TP.Privileges[ 0 ].LUID ) )then
  begin
    if( bEnabled )then
    begin
      TP.Privileges[ 0 ].Attributes  :=
        SE_PRIVILEGE_ENABLED;
    end else
    begin
      TP.Privileges[ 0 ].Attributes  :=
        0;
    end;

    dwRetLen := 0;
    Result := AdjustTokenPrivileges(
                Token,
                False,
                TP,
                SizeOf( TPPrev ),
                TPPrev,
                dwRetLen );
  end;

  CloseHandle( Token );
end;

function TWinSystem.WinExit( iFlags : integer ) : boolean;
begin
  //
  // iFlags:
  //
  //  one of the following must be
  //  specified
  //
  //   EWX_LOGOFF
  //   EWX_REBOOT
  //   EWX_SHUTDOWN
  //
  //  following attributes may be
  //  combined with above flags
  //
  //   EWX_POWEROFF
  //   EWX_FORCE    : terminate processes
  //
  Result := True;
  if( SetPrivilege( 'SeShutdownPrivilege', True ) )then
  begin
    if( not ExitWindowsEx( iFlags, 0 ) )then
    begin
      // handle errors...
      Result := False;
    end;
    SetPrivilege( 'SeShutdownPrivilege', False )
  end else
  begin
    // handle errors...
    Result := False;
  end;
end;
procedure TWinSystem.AppMessage(
  var Msg : TMsg;
  var bHandled : boolean );
begin
  if((WM_SYSCOMMAND = Msg.Message) and
     (SC_SCREENSAVE = Msg.wParam) )then
    bHandled := True;
end;

procedure TWinSystem.RunOnStartup(
  sProgTitle,
  sCmdLine    : string;
  bRunOnce    : boolean );
var
  sKey : string;
  reg  : TRegIniFile;
begin
  if( bRunOnce )then
    sKey := 'Once'
  else
    sKey := '';

  reg := TRegIniFile.Create( '' );
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.WriteString(
    'Software\Microsoft'
    + '\Windows\CurrentVersion\Run'
    + sKey + #0,
    sProgTitle,
    sCmdLine );
  reg.Free;
end;

procedure TWinSystem.RevokeRunOnStartup(sProgTitle: String);
var
  reg  : TRegistry;
begin
  reg := TRegIniFile.Create( '' );
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKey('Software\Microsoft'
    + '\Windows\CurrentVersion\Run', true);
  reg.DeleteValue(sProgTitle);
  reg.CloseKey();
  reg.Free;

end;
function TWinSystem.GetFileSize(filename: String): Longint;
var
    f: File of byte;
    nSize: LongInt;
begin
   Result := 0;
   if FileExists(filename) then  begin
       try
          AssignFile(f,filename);
          Reset(f);
          nSize := FileSize(f);
          CloseFile(f);
          Result:=nSize;
       except
          on EInOutError do HandleIOError;
          else
            Beep;
       end;
   end
end;

function TWinSystem.GetDiskDriveType(sDrive: String): String;
var
    i: integer;
begin
    i := GetDriveType(PChar(sDrive));
    case i of
        0: Result := 'Undetermined.';
        1: Result :=  'The root directory does not exist.';
        DRIVE_REMOVABLE: Result := 'REMOVABLE DRIVE';
        DRIVE_FIXED: Result := 'FIXED DRIVE';
        DRIVE_REMOTE: Result := 'NETWORK DRIVE';
        DRIVE_CDROM: Result := 'CD-ROM DRIVE';
        DRIVE_RAMDISK: Result := 'RAM DISK';
    end;
end;

procedure TWinSystem.DisableScreenSaver();
begin
    SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 0 ,nil, 0);
end;

procedure TWinSystem.EnableScreenSaver();
begin
    SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 1, nil, 0);
end;
procedure TWinSystem.GetFontSize();
begin

  if Screen.PixelsPerInch = 96 then
    FSmallFonts := true
  else if Screen.PixelsPerInch = 120 then
    FSmallFonts := false
  else
    FSmallFonts := false;

end;

procedure TWinSystem.RecycleFiles(Form: TForm; sFiles: String);
var
    op: SHFILEOPSTRUCT;
begin
    ZeroMemory(@op, SizeOf(op));
    op.Wnd := Form.Handle;
    op.pFrom := PChar(sFiles);
    op.wFunc := FO_DELETE;
    op.fFlags := FOF_ALLOWUNDO;
    SHFileOperation(op);
end;

procedure TWinSystem.CopyFlyingFiles(Form: TForm; sFrom: String; sTo: String);
var
    op: SHFILEOPSTRUCT;
begin
    ZeroMemory(@op, SizeOf(op));
    op.Wnd := Form.Handle;
    op.pFrom := PChar(sFrom);
    op.pTo := PChar(sTo);
    op.wFunc := FO_COPY;
    op.fFlags := 0;
    SHFileOperation(op);
end;

procedure TWinSystem.MoveFlyingFiles(Form: TForm; sFrom: String; sTo: String);
var
    op: SHFILEOPSTRUCT;
begin
    ZeroMemory(@op, SizeOf(op));
    op.Wnd := Form.Handle;
    op.pFrom := PChar(sFrom);
    op.pTo := PChar(sTo);
    op.wFunc := FO_MOVE;
    op.fFlags := 0;
    SHFileOperation(op);
end;


procedure TWinSystem.GetExeVersion(sFilename: String);
const
  InfoNum = 10;
  InfoStr : array [1..InfoNum] of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
     'ProductName', 'ProductVersion', 'Comments');
var
  S         : String;
  n, Len, i : Cardinal;
  Buf       : PChar;
  Value     : PChar;
begin
  S := sFilename;
  n := GetFileVersionInfoSize(PChar(S),n);
  if n > 0 then begin
    Buf := AllocMem(n);
    //Memo1.Lines.Add('FileVersionInfoSize='+IntToStr(n));
    GetFileVersionInfo(PChar(S),0,n,Buf);
    for i:=1 to InfoNum do
      if VerQueryValue(Buf,PChar('StringFileInfo\040904E4\'+
                                      InfoStr[i]),Pointer(Value),Len) then
        case i of
            1: FCompanyName := Value;
            2: FFileDescription := Value;
            3: FFileVersion := Value;
            4: FInternalName := Value;
            5: FLegalCopyright := Value;
            6: FLegalTradeMarks := Value;
            7: FOriginalFilename := Value;
            8: FProductName := Value;
            9: FProductVersion := Value;
            10: FComments := Value;
        end;
    FreeMem(Buf,n);
  end
  else
  begin
             FCompanyName := 'Not available';
             FFileDescription := 'Not available';
             FFileVersion := 'Not available';
             FInternalName := 'Not available';
             FLegalCopyright := 'Not available';
             FLegalTradeMarks := 'Not available';
             FOriginalFilename := 'Not available';
             FProductName := 'Not available';
             FProductVersion := 'Not available';
             FComments := 'Not available';

  end;

end;

procedure TWinSystem.HandleIOError();
begin
    MessageBox(0, 'The selected file is currently under use', 'File is used', MB_ICONSTOP);
end;
end.
